@@ -14,7 +14,7 @@ from flask_caching.backends.base import BaseCache, iteritems_wrapper
 try:
     import cPickle as pickle
 except ImportError:  # pragma: no cover
-    import pickle
+    import pickle  # type: ignore
 
 
 class RedisCache(BaseCache):
@@ -49,7 +49,7 @@ class RedisCache(BaseCache):
         key_prefix=None,
         **kwargs
     ):
-        super(RedisCache, self).__init__(default_timeout)
+        super().__init__(default_timeout)
         if host is None:
             raise ValueError("RedisCache host parameter may not be None")
         if isinstance(host, str):
@@ -70,6 +70,39 @@ class RedisCache(BaseCache):
         self._write_client = self._read_clients = client
         self.key_prefix = key_prefix or ""
 
+    @classmethod
+    def factory(cls, app, config, args, kwargs):
+        try:
+            from redis import from_url as redis_from_url
+        except ImportError:
+            raise RuntimeError("no redis module found")
+
+        kwargs.update(
+            dict(
+                host=config.get("CACHE_REDIS_HOST", "localhost"),
+                port=config.get("CACHE_REDIS_PORT", 6379),
+            )
+        )
+        password = config.get("CACHE_REDIS_PASSWORD")
+        if password:
+            kwargs["password"] = password
+
+        key_prefix = config.get("CACHE_KEY_PREFIX")
+        if key_prefix:
+            kwargs["key_prefix"] = key_prefix
+
+        db_number = config.get("CACHE_REDIS_DB")
+        if db_number:
+            kwargs["db"] = db_number
+
+        redis_url = config.get("CACHE_REDIS_URL")
+        if redis_url:
+            kwargs["host"] = redis_from_url(
+                redis_url, db=kwargs.pop("db", None)
+            )
+
+        return cls(*args, **kwargs)
+
     def _get_prefix(self):
         return (
             self.key_prefix
@@ -135,11 +168,14 @@ class RedisCache(BaseCache):
     def add(self, key, value, timeout=None):
         timeout = self._normalize_timeout(timeout)
         dump = self.dump_object(value)
-        return self._write_client.setnx(
+        created = self._write_client.setnx(
             name=self._get_prefix() + key, value=dump
-        ) and self._write_client.expire(
-            name=self._get_prefix() + key, time=timeout
         )
+        if created and timeout != -1:
+            self._write_client.expire(
+                name=self._get_prefix() + key, time=timeout
+            )
+        return created
 
     def set_many(self, mapping, timeout=None):
         timeout = self._normalize_timeout(timeout)
@@ -191,8 +227,7 @@ class RedisCache(BaseCache):
         )
 
     def unlink(self, *keys):
-        """when redis-py >= 3.0.0 and redis > 4, support this operation
-        """
+        """when redis-py >= 3.0.0 and redis > 4, support this operation"""
         if not keys:
             return
         if self.key_prefix:
@@ -237,7 +272,7 @@ class RedisSentinelCache(RedisCache):
         key_prefix=None,
         **kwargs
     ):
-        super(RedisSentinelCache, self).__init__(default_timeout)
+        super().__init__(default_timeout=default_timeout)
 
         try:
             import redis.sentinel
@@ -273,3 +308,102 @@ class RedisSentinelCache(RedisCache):
         self._read_clients = sentinel.slave_for(master)
 
         self.key_prefix = key_prefix or ""
+
+    @classmethod
+    def factory(cls, app, config, args, kwargs):
+        kwargs.update(
+            dict(
+                sentinels=config.get(
+                    "CACHE_REDIS_SENTINELS", [("127.0.0.1", 26379)]
+                ),
+                master=config.get("CACHE_REDIS_SENTINEL_MASTER", "mymaster"),
+                password=config.get("CACHE_REDIS_PASSWORD", None),
+                sentinel_password=config.get(
+                    "CACHE_REDIS_SENTINEL_PASSWORD", None
+                ),
+                key_prefix=config.get("CACHE_KEY_PREFIX", None),
+                db=config.get("CACHE_REDIS_DB", 0),
+            )
+        )
+
+        return cls(*args, **kwargs)
+
+
+class RedisClusterCache(RedisCache):
+    """Uses the Redis key-value store as a cache backend.
+
+    The first argument can be either a string denoting address of the Redis
+    server or an object resembling an instance of a rediscluster.RedisCluster
+    class.
+
+    Note: Python Redis API already takes care of encoding unicode strings on
+    the fly.
+
+
+    :param cluster: The redis cluster nodes address separated by comma.
+                    e.g. host1:port1,host2:port2,host3:port3 .
+    :param password: password authentication for the Redis server.
+    :param default_timeout: the default timeout that is used if no timeout is
+                            specified on :meth:`~BaseCache.set`. A timeout of
+                            0 indicates that the cache never expires.
+    :param key_prefix: A prefix that should be added to all keys.
+
+    Any additional keyword arguments will be passed to
+    ``rediscluster.RedisCluster``.
+    """
+
+    def __init__(
+        self,
+        cluster="",
+        password="",
+        default_timeout=300,
+        key_prefix="",
+        **kwargs
+    ):
+        super().__init__(default_timeout=default_timeout)
+
+        if kwargs.get("decode_responses", None):
+            raise ValueError(
+                "decode_responses is not supported by " "RedisCache."
+            )
+
+        try:
+            from rediscluster import RedisCluster
+        except ImportError:
+            raise RuntimeError("no rediscluster module found")
+
+        try:
+            nodes = [(node.split(":")) for node in cluster.split(",")]
+            startup_nodes = [
+                {"host": node[0].strip(), "port": node[1].strip()}
+                for node in nodes
+            ]
+        except IndexError:
+            raise ValueError(
+                "Please give the correct cluster argument "
+                "e.g. host1:port1,host2:port2,host3:port3"
+            )
+        # Skips the check of cluster-require-full-coverage config,
+        # useful for clusters without the CONFIG command (like aws)
+        skip_full_coverage_check = kwargs.pop("skip_full_coverage_check", True)
+
+        cluster = RedisCluster(
+            startup_nodes=startup_nodes,
+            password=password,
+            skip_full_coverage_check=skip_full_coverage_check,
+            **kwargs
+        )
+        self._write_client = self._read_clients = cluster
+        self.key_prefix = key_prefix
+
+    @classmethod
+    def factory(cls, app, config, args, kwargs):
+        kwargs.update(
+            dict(
+                cluster=config.get("CACHE_REDIS_CLUSTER", ""),
+                password=config.get("CACHE_REDIS_PASSWORD", ""),
+                default_timeout=config.get("CACHE_DEFAULT_TIMEOUT", 300),
+                key_prefix=config.get("CACHE_KEY_PREFIX", ""),
+            )
+        )
+        return cls(*args, **kwargs)
